@@ -1,6 +1,6 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
@@ -554,7 +554,7 @@ int iso_utf8_2gsm(char* source, int size, char* destination, int max)
 
     if (found==0)
     {
-      writelogfile0(LOG_NOTICE, process_title,
+      writelogfile0(LOG_NOTICE, 0,
         tb_sprintf("Cannot convert %i. character %c 0x%2X to GSM, you might need to update the translation tables.",
         source_count +1, source[source_count], source[source_count]));
 #ifdef DEBUGMSG
@@ -624,7 +624,7 @@ int iso2utf8_file(FILE *fp, char *ascii, int userdatalength)
     {
       if (log_charconv)
         logch(NULL);
-      writelogfile0(LOG_NOTICE, process_title,
+      writelogfile0(LOG_NOTICE, 0,
         tb_sprintf("UTF-8 conversion error with %i. ch 0x%2X %c.", idx +1, c, (char)c));
 #ifdef DEBUGMSG
   printf("%s\n", tb);
@@ -648,7 +648,7 @@ int iso2utf8_file(FILE *fp, char *ascii, int userdatalength)
       {
         if (log_charconv)
           logch(NULL);
-        writelogfile0(LOG_NOTICE, process_title, tb_sprintf("Fatal file write error in UTF-8 conversion"));
+        writelogfile0(LOG_NOTICE, 0, tb_sprintf("Fatal file write error in UTF-8 conversion"));
 #ifdef DEBUGMSG
   printf("%s\n", tb);
 #endif
@@ -663,68 +663,6 @@ int iso2utf8_file(FILE *fp, char *ascii, int userdatalength)
   return result;
 }
 
-/*
-int iso2gsm(char* source, int size, char* destination, int max)
-{
-  int table_row;
-  int source_count=0;
-  int dest_count=0;
-  int found=0;
-
-  destination[dest_count]=0;
-  if (source==0 || size==0)
-    return 0;
-  // Convert each character untl end of string
-  while (source_count<size && dest_count<max)
-  {
-    // search in normal translation table
-    found=0;
-    table_row=0;
-    while (charset[table_row*2])
-    {
-      if (charset[table_row*2]==source[source_count])
-      {
-	destination[dest_count++]=charset[table_row*2+1];
- 	found=1;
-	break;
-      }
-      table_row++;
-    }
-    // if not found in normal table, then search in the extended table
-    if (found==0)
-    {
-      table_row=0;
-      while (ext_charset[table_row*2])
-      {
-        if (ext_charset[table_row*2]==source[source_count])
-        {
-          destination[dest_count++]=0x1B;
-          if (dest_count<max)
-	    destination[dest_count++]=ext_charset[table_row*2+1];
-	  found=1;
-	  break;
-	}
-	table_row++;
-      }
-    }
-    // if also not found in the extended table, then log a note
-    if (found==0)
-    {
-      writelogfile0(LOG_NOTICE, process_title,
-        tb_sprintf("Cannot convert %i. ISO character %c 0x%2X to GSM, you might need to update the translation tables.",
-        source_count +1, source[source_count], source[source_count]));
-#ifdef DEBUGMSG
-  printf("%s\n", tb);
-#endif
-    }
-    source_count++;
-  }
-  // Terminate destination string with 0, however 0x00 are also allowed within the string.
-  destination[dest_count]=0;
-  return dest_count;
-}
-*/
-
 int gsm2iso(char* source, int size, char* destination, int max)
 {
   int source_count=0;
@@ -749,7 +687,7 @@ int gsm2iso(char* source, int size, char* destination, int max)
         destination[dest_count++] = (char)GSM_CURRENCY_SYMBOL_TO_ISO;
       else
       {
-        writelogfile0(LOG_NOTICE, process_title,
+        writelogfile0(LOG_NOTICE, 0,
           tb_sprintf("Cannot convert GSM character 0x%2X to ISO, you might need to update the 1st translation table.",
           source[source_count]));
 #ifdef DEBUGMSG
@@ -764,7 +702,7 @@ int gsm2iso(char* source, int size, char* destination, int max)
         destination[dest_count++] = newch;
       else
       {
-        writelogfile0(LOG_NOTICE, process_title,
+        writelogfile0(LOG_NOTICE, 0,
           tb_sprintf("Cannot convert extended GSM character 0x1B 0x%2X, you might need to update the 2nd translation table.",
           source[source_count]));
 #ifdef DEBUGMSG
@@ -779,49 +717,11 @@ int gsm2iso(char* source, int size, char* destination, int max)
   return dest_count;
 }
 
-/*
-int unicode2sms(char* source, int size, char* destination, int max)
-{
-  int foundstart=0;
-  int position=1;
-  unsigned char previous='\0';
-  int source_count=0;
-  int dest_count=0;
-
-  if (source==0 || size==0)
-  {
-    destination[0]=0;
-    return 0;
-  }
-
-  while (source_count<size && dest_count<max)
-  {
-    if (position == 1)
-    {
-      if (foundstart == 1)
-        destination[dest_count++] = source[source_count];
-      position = 2;
-    }
-    else
-    {
-      if ((unsigned char)source[source_count] != 0xFF)
-        destination[dest_count++] = source[source_count];
-      if (foundstart == 0 && source[source_count] == 0x0A && previous == 0x0A)
-        foundstart = 1;
-      previous = source[source_count];
-      position = 1;
-    }
-    source_count++;
-  }
-  destination[dest_count]=0;
-  return dest_count;
-}
-*/
-
 int decode_ucs2(char *buffer, int len)
 {
   int i;
   char *d = buffer;
+  int j;
 
   for (i = 0; i < len; )
   {
@@ -840,6 +740,12 @@ int decode_ucs2(char *buffer, int len)
   }
   i = (d -buffer) +len;
   *(buffer +i) = '\0';
+
+  // 3.1.6: Fix euro character(s):
+  for (j = 0; buffer[j]; j++)
+    if (buffer[j] == (char) 0xAC)
+      buffer[j] = (char) 0xA4;
+
   return i;
 }
 
@@ -886,7 +792,7 @@ void logch(char* format, ...)
   if (flush)
   {
     if (*logch_buffer)
-      writelogfile(LOG_DEBUG, process_title, "charconv: %s", logch_buffer);
+      writelogfile(LOG_DEBUG, 0, "charconv: %s", logch_buffer);
     *logch_buffer = 0;
 #ifdef DEBUGMSG
   printf("\n");
