@@ -1,6 +1,6 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
@@ -13,6 +13,7 @@ the GNU General Public License as published by the Free Software Foundation.
 Either version 2 of the License, or (at your option) any later version.
 */
 
+#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
@@ -26,6 +27,7 @@ Either version 2 of the License, or (at your option) any later version.
 #include <sys/wait.h>
 #include <time.h>
 #include <syslog.h>
+#include <stdarg.h>
 #include "extras.h"
 #include "locking.h"
 #include "smsd_cfg.h"
@@ -174,8 +176,10 @@ int movefile( char*  filename,  char*  directory)
     dest=open(newname,O_WRONLY|O_CREAT|O_TRUNC,statbuf.st_mode);
     if (dest>=0)
     {
-      while ((readcount=read(source,&storage,sizeof(storage)))>0)
-        if (write(dest,&storage,readcount)<readcount)
+      //while ((readcount=read(source,&storage,sizeof(storage)))>0)
+      //  if (write(dest,&storage,readcount)<readcount)
+      while ((readcount = read(source, storage, sizeof(storage))) > 0)
+        if (write(dest, storage, readcount) < readcount)
 	{
 	  close(dest);
 	  close(source);
@@ -196,35 +200,6 @@ int movefile( char*  filename,  char*  directory)
     return 0;
 }
 
-// 3.0.9: Return values:
-// 0 = OK.
-// 1 = lockfile cannot be created. It exists.
-// 2 = file copying failed.
-// 3 = lockfile removing failed.
-int movefilewithdestlock( char*  filename,  char*  directory)
-{
-  char lockfilename[PATH_MAX];
-  char* cp;
-
-  //create lockfilename in destination
-  cp=strrchr(filename,'/');
-  if (cp)
-    sprintf(lockfilename,"%s%s",directory,cp);
-  else
-    sprintf(lockfilename,"%s/%s",directory,filename);
-  //create lock and move file
-  if (!lockfile(lockfilename))
-    return 1;
-  if (!movefile(filename,directory))
-  {
-    unlockfile(lockfilename);
-    return 2;
-  }
-  if (!unlockfile(lockfilename))
-    return 3;
-  return 0;
-}
-
 // 3.1beta7: Return values:
 // 0 = OK.
 // 1 = lockfile cannot be created. It exists.
@@ -369,7 +344,7 @@ char *cutspaces(char *text)
   return text;
 }
 
-void cut_emptylines(char*  text)
+char *cut_emptylines(char *text)
 {
   char* posi;
   char* found;
@@ -382,6 +357,7 @@ void cut_emptylines(char*  text)
     else
       posi++;
   }
+  return text;
 }
 
 int is_number( char*  text)
@@ -405,10 +381,13 @@ int is_highpriority(char *filename)
   extern const char *HDR_Priority;
   extern char HDR_Priority2[];
   int hlen;
-  int hlen2;
+  int hlen2 = 0;
   char *compare;
   char *compare2 = 0;
 
+  if (ignore_outgoing_priority)
+    return 0;
+
   // get_header() and test_header() can be moved to this file,
   // but this is faster:
   if (*HDR_Priority2 && strcmp(HDR_Priority2, "-"))
@@ -501,6 +480,9 @@ int getpdufile(char *filename)
                 }
               }
             }
+            // 3.1.1:
+            //close the directory. added by Callan Fox to fix open handles problem
+            closedir(dirdata);
           }
         }
       }
@@ -521,6 +503,8 @@ int getfile(char *dir, char *filename)
   char tmpname[PATH_MAX];
   int found_highpriority = 0;
   int i;
+  char storage_key[PATH_MAX +3];
+
 
 #ifdef DEBUGMSG
   printf("!! getfile(dir=%s, ...)\n", dir);
@@ -531,8 +515,8 @@ int getfile(char *dir, char *filename)
   if (!(dirdata = opendir(dir)))
   {
     // Something has happened to dir after startup check was done successfully.
-    writelogfile0(LOG_CRIT, process_title, tb_sprintf("Stopping. Cannot open dir %s %s", dir, strerror(errno)));
-    alarm_handler0(LOG_CRIT, process_title, tb);
+    writelogfile0(LOG_CRIT, 0, tb_sprintf("Stopping. Cannot open dir %s %s", dir, strerror(errno)));
+    alarm_handler0(LOG_CRIT, tb);
     abnormal_termination(1);
   }
   else
@@ -548,10 +532,8 @@ int getfile(char *dir, char *filename)
       {
         if (strcmp(tmpname +strlen(tmpname) -5, ".LOCK") != 0)
         {
-          if (!islocked(tmpname))
+          if (trust_outgoing || !islocked(tmpname))
           {
-            char storage_key[PATH_MAX +3];
-
             sprintf(storage_key, "*%s*\n", tmpname);
 
             // 3.1beta7, 3.0.10:
@@ -559,8 +541,7 @@ int getfile(char *dir, char *filename)
               if (!check_access(tmpname))
                 chmod(tmpname, 0766);
 
-            // 3.1:
-            if (!os_cygwin && !file_is_writable(tmpname))
+            if (!trust_outgoing && !os_cygwin && !file_is_writable(tmpname))
             {
               int result = 1;
               char tmp_filename[PATH_MAX +7];
@@ -599,16 +580,14 @@ int getfile(char *dir, char *filename)
               }
             }
 
-            if (!file_is_writable(tmpname))
+            if (!trust_outgoing && !file_is_writable(tmpname))
             {
               int report = 1;
               char reason[100];
 
-              //if (access(tmpname, R_OK | W_OK) != 0)
               if (!check_access(tmpname))
               {
-                //snprintf(reason, sizeof(reason), "%i %s", errno, strerror(errno));
-                snprintf(reason, sizeof(reason), "%s", "Access denied.");
+                snprintf(reason, sizeof(reason), "%s", "Access denied. Check the file and directory permissions.");
                 if (getfile_err_store)
                   if (strstr(getfile_err_store, storage_key))
                     report = 0; 
@@ -626,10 +605,17 @@ int getfile(char *dir, char *filename)
                   if (getfile_err_store)
                     strcat(getfile_err_store, storage_key);
 
-                  writelogfile0(LOG_ERR, process_title, tb_sprintf("Cannot handle %s: %s", tmpname, reason));
-                  alarm_handler0(LOG_ERR, process_title, tb);
+                  writelogfile0(LOG_ERR, 0, tb_sprintf("Cannot handle %s: %s", tmpname, reason));
+                  alarm_handler0(LOG_ERR, tb);
                 }
               }
+              else
+              {
+                // 3.1.5: This error is repeated:
+                snprintf(reason, sizeof(reason), "%s", "Dont know why. Check the file and directory permissions.");
+                writelogfile0(LOG_ERR, 0, tb_sprintf("Cannot handle %s: %s", tmpname, reason));
+                alarm_handler0(LOG_ERR, tb);
+              }
             }
             else
             {
@@ -656,6 +642,7 @@ int getfile(char *dir, char *filename)
 #endif
                 continue;
               }
+
               if (i && !found_highpriority)
               {
                 // Forget possible previous found normal priority file:
@@ -666,6 +653,12 @@ int getfile(char *dir, char *filename)
 #ifdef DEBUGMSG
   printf("**%s %s %i ", dir, ent->d_name, (int)(statbuf.st_mtime));
 #endif
+
+              // 3.1.6: Files with the same timestamp: compare names:
+              if (found && statbuf.st_mtime == mtime)
+                if (strcmp(fname, tmpname) > 0)
+                  mtime = 0;
+
               if (mtime == 0 || statbuf.st_mtime < mtime)
               {
 #ifdef DEBUGMSG
@@ -727,7 +720,7 @@ int getfile(char *dir, char *filename)
     printf("!! process: %i, getfile_err_store:\n%s", process_id, getfile_err_store);
 #endif
 
-    if (found)
+    if (!trust_outgoing && found)
     {
       /* check if the file grows at the moment (another program writes to it) */
       int groesse1;
@@ -755,60 +748,100 @@ int getfile(char *dir, char *filename)
   return found;
 }
 
-int my_system( char*  command, char *info)
+int my_system(char *command, char *info)
 {
-  int pid,status;
+  int pid;
+  int status;
+  time_t start_time;
 
+  start_time = time(0);
 #ifdef DEBUGMSG
-  printf("!! my_system(%s, %s)\n",command, info);
+  printf("!! my_system(%s, %s)\n", command, info);
 #endif
-  if (command==0)
-    return 1;
-  pid=fork();
-  if (pid==-1)
+  writelogfile0(LOG_DEBUG, 0, tb_sprintf("Running %s: %s", info, command));
+
+  pid = fork();
+  if (pid == -1)
+  {
+    writelogfile0(LOG_CRIT, 0, tb_sprintf("Fatal error: fork failed."));
     return -1;
-  if (pid==0)  // only executed in the child
+  }
+
+  if (pid == 0) // only executed in the child
   {
-    char* argv[4];
+    char *argv[4];
+    char *p;
 
 #ifdef DEBUGMSG
-    printf("!! pid=%i, child running external command\n",pid);
+  printf("!! pid=%i, child running external command\n", pid);
 #endif
-    argv[0]="sh";
-    argv[1]="-c";
-    argv[2]=(char*) command;
-    argv[3]=0;
-    execv("/bin/sh",argv);  // replace child with the external command
+
+    // TODO: sh still ok?
+    argv[0] = "sh";
+    if ((p = strrchr(shell, '/')))
+      argv[0] = p +1;
+
+    argv[1] = "-c";
+    argv[2] = command; //(char*) command;
+    argv[3] = 0;
+
+    // 3.1.5:
+    //execv("/bin/sh",argv);  // replace child with the external command
+    execv(shell, argv); // replace child with the external command
+    writelogfile0(LOG_CRIT, 1, tb_sprintf("Fatal error: execv( %s ) returned: %i, %s", shell, errno, strerror(errno)));
+
 #ifdef DEBUGMSG
-    printf("!! pid=%i, execv() failed, child exits now\n",pid);
+  printf("!! pid=%i, execv() failed, %i, %s\n", pid, errno, strerror(errno));
+  printf("!! child exits now\n");
 #endif
-    exit(1);                // exit with error when the execv call failed
+    exit((errno)? errno : 1); // exit with error when the execv call failed
   }
-  errno=0;
+
+  errno = 0;
 #ifdef DEBUGMSG
-    printf("!! father waiting for child %i\n",pid);
+  printf("!! father waiting for child %i\n", pid);
 #endif
-  *run_info = 0;
-  if (info)
-    if (*info)
-      strcpy(run_info, info);
-  do
+  strcpy(run_info, info);
+
+  while (1)
   {
-    if (waitpid(pid,&status,0)==-1)
+    if (waitpid(pid, &status, 0) == -1)
     {
-      if (errno!=EINTR)
+      if (errno != EINTR)
       {
         *run_info = 0;
+        writelogfile0(LOG_ERR, 0, tb_sprintf("Done: %s, execution time %i sec., errno: %i, %s", info, time(0) -start_time, errno, strerror(errno)));
         return -1;
       }
     }
     else
     {
+      int level = LOG_DEBUG;
+      int trouble = 0;
+
       *run_info = 0;
+
+      // 3.1.6: When running checkhandler and it spooled a message, return value 2 SHOULD NOT activate trouble logging:
+      //writelogfile0((status == 0) ? LOG_DEBUG : LOG_ERR, (status == 0) ? 0 : 1, tb_sprintf("Done: %s, execution time %i sec., status: %i", info, time(0) - start_time, status));
+      if (!strcmp(info, "checkhandler"))
+      {
+        if (status != 0 && WEXITSTATUS(status) != 2)
+        {
+          level = LOG_ERR;
+          trouble = 1;
+        }
+      }
+      else if (status != 0)
+      {
+        level = LOG_ERR;
+        trouble = 1;
+      }
+
+      writelogfile0(level, trouble, tb_sprintf("Done: %s, execution time %i sec., status: %i (%i)", info, time(0) - start_time, status, WEXITSTATUS(status)));
+
       return WEXITSTATUS(status);
     }
   }
-  while (1);    
 }
 
 int write_pid( char* filename)
@@ -827,9 +860,30 @@ int write_pid( char* filename)
   return 0;
 }
 
+int check_pid(char *filename)
+{
+  int result = 0;
+  char pid[20];
+  FILE *fp;
+  char buffer[256];
+
+  sprintf(pid,"%i\n", (int)getpid());
+  if ((fp = fopen(filename, "r")))
+  {
+    if (fgets(buffer, sizeof(buffer), fp))
+      if (strcmp(pid, buffer))
+        result = atoi(buffer);
+
+    fclose(fp);
+  }
+
+  return result;
+}
+
 void remove_pid( char* filename)
 {
-  unlink(filename);
+  if (*filename)
+    unlink(filename);
 }
 
 int parse_validity(char *value, int defaultvalue)
@@ -1062,7 +1116,6 @@ int is_executable(char *filename)
 int check_access(char *filename)
 {
   // access() migth do this easier, but in Gygwin it returns 0 even when requested permissions are NOT granted.
-  // TODO: should be combined with is_executable.
   int result = 0;
   struct stat statbuf;
   mode_t mode;
@@ -1108,3 +1161,65 @@ int check_access(char *filename)
 
   return result;
 }
+
+int value_in(int value, int arg_count, ...)
+{
+  int result = 0;
+  va_list ap;
+
+  va_start(ap, arg_count);
+  for (; arg_count > 0; arg_count--)
+    if (value == va_arg(ap, int))
+      result = 1;
+
+  va_end(ap);
+
+  return result;
+}
+
+int t_sleep(int seconds)
+{
+  int i;
+
+  for (i = 0; i < seconds; i++)
+  {
+    if (terminate == 1)
+      return 1;
+    sleep(1);
+  }
+  return 0;
+}
+
+int usleep_until(unsigned long long target_time)
+{
+  struct timeval tv;
+  struct timezone tz;
+  unsigned long long now;
+
+  do
+  {
+    gettimeofday(&tv, &tz);
+    now = (unsigned long long)tv.tv_sec *1000000 +tv.tv_usec;
+
+    if (terminate == 1)
+      return 1;
+
+    if (now < target_time)
+      usleep(100000);
+  }
+  while (now < target_time);
+
+  return 0;
+}
+
+unsigned long long time_usec()
+{
+  struct timeval tv;
+  struct timezone tz;
+  //struct tm *tm;
+
+  gettimeofday(&tv, &tz);
+  /*tm =*/ //localtime(&tv.tv_sec);
+
+  return (unsigned long long)tv.tv_sec *1000000 +tv.tv_usec;
+}
