@@ -1,6 +1,6 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
@@ -40,7 +40,7 @@ int movefile(char* filename, char* directory);
 
 /* Moves a file into another directory. Destination file is protected with
    a lock file during the operation. Returns 1 if success. */
-int movefilewithdestlock(char* filename, char* directory);
+//int movefilewithdestlock(char* filename, char* directory);
 int movefilewithdestlock_new(char* filename, char* directory, int keep_fname, int store_original_fname, char *prefix, char *newfilename);
 
 /* removes ctrl chars at the beginning and the end of the text and removes */
@@ -48,7 +48,7 @@ int movefilewithdestlock_new(char* filename, char* directory, int keep_fname, in
 char *cutspaces(char *text);
 
 /* removes all empty lines */
-void cut_emptylines(char* text);
+char *cut_emptylines(char *text);
 
 /* Checks if the text contains only numbers. */
 int is_number(char* text);
@@ -62,10 +62,11 @@ int getpdufile(char *filename);
 int getfile(char* dir, char* filename);
 
 /* Replacement for system() wich can be breaked. See man page of system() */
-int my_system(char* command, char *info);
+int my_system(char *command, char *info);
 
 /* Create and remove a PID file */
 int write_pid(char* filename);
+int check_pid(char *filename);
 void remove_pid(char* filename);
 
 /* Parse validity value string */
@@ -79,4 +80,13 @@ int getrand(int toprange);
 int is_executable(char *filename);
 int check_access(char *filename);
 
+int value_in(int value, int arg_count, ...);
+
+// t_sleep returns 1 if terminate is set to 1 while sleeping:
+int t_sleep(int seconds);
+
+int usleep_until(unsigned long long target_time);
+
+unsigned long long time_usec();
+
 #endif
