@@ -1,6 +1,6 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
@@ -42,7 +42,7 @@ int lockfile( char*  filename)
     if (lockfile>=0)
     {
       sprintf(pid,"%i\n",(int)getpid());
-      write(lockfile,&pid,strlen(pid));
+      write(lockfile, pid, strlen(pid));
       close(lockfile);
       return 1;
     }
