@@ -1,6 +1,6 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
@@ -16,7 +16,9 @@ Either version 2 of the License, or (at your option) any later version.
 #ifndef LOGGING_H
 #define LOGGING_H
 
-int openlogfile(char* filename,char* progname,int facility,int level);
+int trouble_logging_started;
+
+int openlogfile(char *filename, int facility, int level);
 
 // if filename if 0, "" or "syslog": opens syslog. Level is ignored.
 // else: opens a log file. Facility is not used. Level specifies the verbosity (9=highest).
@@ -26,12 +28,8 @@ int openlogfile(char* filename,char* progname,int facility,int level);
 
 
 void closelogfile();
-
-// closes a log file.
-
-void writelogfile0(int severity, char *progname, char *text);
-void writelogfile(int severity,char* progname,char* format, ...);
-
-// write to a log file. 2nd and following arguments are like in printf.
+void writelogfile0(int severity, int trouble, char *text);
+void writelogfile(int severity, int trouble, char* format, ...);
+void flush_smart_logging();
 
 #endif
