@@ -1,6 +1,6 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
@@ -48,7 +48,7 @@ int add_warning(char *buffer, char *format, ...)
     else
     {
       result = 0;
-      writelogfile(LOG_ERR, process_title, "PDU %s%s", title, text);
+      writelogfile(LOG_ERR, 1, "PDU %s%s", title, text);
     }
   }
 
@@ -215,6 +215,75 @@ void binary2pdu(char* binary, int length, char* pdu)
   }
 }
 
+int set_numberformat(int *numberformat, char *number, int number_type)
+{
+  int nf = NF_INTERNATIONAL;
+  char *p;
+
+  if (numberformat)
+    nf = *numberformat;
+
+  if (nf == NF_INTERNATIONAL)
+  {
+    // If international prefixes are defined, international format is used if a number matches to the list.
+    // If it does not match, national format will be used.
+    if (*international_prefixes)
+    {
+      p = international_prefixes;
+      do
+      {
+        if (!strncmp(number, p, strlen(p)))
+          break;
+        p += strlen(p) +1;
+      }
+      while (*p);
+
+      if (!(*p))
+        nf = NF_NATIONAL;
+    }
+    
+    if (nf == NF_INTERNATIONAL)
+    {
+      // If national prefixes are defined, national format is used if a number matches to the list.
+      if (*national_prefixes)
+      {
+        p = national_prefixes;
+        do
+        {
+          if (!strncmp(number, p, strlen(p)))
+          {
+            nf = NF_NATIONAL;
+            break;
+          }
+          p += strlen(p) +1;
+        }
+        while (*p);
+      }
+    }
+  }
+
+  // Finally, user setting overrides all...
+  switch (number_type)
+  {
+    case 0: // "unknown"
+      nf = NF_UNKNOWN; // 1000 0001
+      break;
+
+    case 1: // "international"
+      nf = NF_INTERNATIONAL; // 1001 0001
+      break;
+
+    case 2: // "national"
+      nf = NF_NATIONAL; // 1010 0001
+      break;
+  }
+
+  if (numberformat)
+    *numberformat = nf;
+
+  return nf;
+}
+
 // Make the PDU string from a mesage text and destination phone number.
 // The destination variable pdu has to be big enough. 
 // alphabet indicates the character set of the message.
@@ -223,7 +292,7 @@ void binary2pdu(char* binary, int length, char* pdu)
 // if udh is true, then udh_data contains the optional user data header in hex dump, example: "05 00 03 AF 02 01"
 
 void make_pdu(char* number, char* message, int messagelen, int alphabet, int flash_sms, int report, int with_udh,
-              char* udh_data, char* mode, char* pdu, int validity, int replace_msg, int system_msg)
+              char* udh_data, char* mode, char* pdu, int validity, int replace_msg, int system_msg, int number_type)
 {
   int coding;
   int flags;
@@ -234,16 +303,19 @@ void make_pdu(char* number, char* message, int messagelen, int alphabet, int fla
   char *p;
   int l;
 
-  if (number[0]=='s')  // Is number starts with s, then send it without number format indicator
+  if (number[0] == 's')  // Is number starts with s, then send it without number format indicator
   {
-    numberformat=129;
-    strcpy(tmp,number+1);
+    numberformat = NF_UNKNOWN;
+    strcpy(tmp, number +1);
   }
   else
   {
-    numberformat=145;
-    strcpy(tmp,number);
+    numberformat = NF_INTERNATIONAL;
+    strcpy(tmp, number);
   }
+
+  set_numberformat(&numberformat, tmp, number_type);
+
   numberlength=strlen(tmp);
   // terminate the number with F if the length is odd
   if (numberlength%2)
@@ -807,7 +879,7 @@ int pdu2binary(char* pdu, char* binary, int *data_length, int *expected_length,
 int explain_toa(char *dest, char *octet_char, int octet_int)
 {
   int result;
-  char *p;
+  char *p = "reserved";
 
   if (octet_char)
     result = octet2bin_check(octet_char);
@@ -825,7 +897,7 @@ int explain_toa(char *dest, char *octet_char, int octet_int)
       case 4: p = "subsciber"; break;
       case 5: p = "alphanumeric"; break;
       case 6: p = "abbreviated"; break;
-      case 7: p = "reserved"; break;
+      //case 7: p = "reserved"; break;
     }
 
     if (octet_char)
@@ -842,7 +914,7 @@ int explain_toa(char *dest, char *octet_char, int octet_int)
       case 8: p = "national"; break;
       case 9: p = "private"; break;
       case 10: p = "ERMES"; break;
-      default: p = "reserved"; break;
+      //default: p = "reserved"; break;
     }
     sprintf(strchr(dest, 0), ", %s", p);
   }
@@ -857,13 +929,15 @@ int explain_toa(char *dest, char *octet_char, int octet_int)
 // sendr Sender
 // date and time Date/Time-stamp
 // message the message text or binary data
+// bin_udh: 1 if udh is taken from the PDU with binary messages too
 // returns length of message
 int split_type_0(char *full_pdu, char* Src_Pointer, int* alphabet, char* sendr, char* date, char* time,
                  char* message, int *message_length, int *expected_length, int with_udh, char* udh_data,
-                 char *udh_type, char *from_toa, int *replace, char **err_str, char *warning_headers, int *flash)
+                 char *udh_type, char *from_toa, int *replace, char **err_str, char *warning_headers,
+                 int *flash, int bin_udh)
 {
   int Length;
-  int padding;
+  int padding = 0;
   char tmpsender[100];
   int result = 0;
   int i;
@@ -879,9 +953,13 @@ int split_type_0(char *full_pdu, char* Src_Pointer, int* alphabet, char* sendr,
   else
   {
     Length = octet2bin_check(Src_Pointer);
-    if (Length < 1 || Length > MAX_ADDRESS_LENGTH)
+    // 3.1.5: Sender address length can be zero. There is still address type octet (80).
+    if (Length < 0 || Length > MAX_ADDRESS_LENGTH)
       pdu_error(err_str, 0, Src_Pointer -full_pdu, 2, "Invalid sender address length: \"%.2s\"", Src_Pointer);
     else
+    if (Length == 0)
+      Src_Pointer += 4;
+    else
     {
       padding=Length%2;
       Src_Pointer+=2;
@@ -940,7 +1018,7 @@ int split_type_0(char *full_pdu, char* Src_Pointer, int* alphabet, char* sendr,
               {
                 // Not a fatal error (?)
                 //pdu_error(err_str, 0, Src_Pointer -full_pdu, Length +padding, "Invalid character(s) in sender address: \"%s\"", sendr);
-                //*sendr = 0;
+                // *sendr = 0;
                 add_warning(warning_headers, "Invalid character(s) in sender address.");
                 break;
               }
@@ -999,7 +1077,7 @@ int split_type_0(char *full_pdu, char* Src_Pointer, int* alphabet, char* sendr,
               {
                 // Not a fatal error (?)
                 //pdu_error(err_str, 0, Src_Pointer -full_pdu, 6, "Invalid value(s) in date of Service Centre Time Stamp: \"%s\"", date);
-                //*date = 0;
+                // *date = 0;
                 add_warning(warning_headers, "Invalid values(s) in date of Service Centre Time Stamp.");
               }
 
@@ -1014,7 +1092,7 @@ int split_type_0(char *full_pdu, char* Src_Pointer, int* alphabet, char* sendr,
               {
                 // Not a fatal error (?)
                 //pdu_error(err_str, 0, Src_Pointer -full_pdu, 6, "Invalid value(s) in time of Service Centre Time Stamp: \"%s\"", time);
-                //*time = 0;
+                // *time = 0;
                 add_warning(warning_headers, "Invalid values(s) in time of Service Centre Time Stamp.");
               }
 
@@ -1049,8 +1127,10 @@ int split_type_0(char *full_pdu, char* Src_Pointer, int* alphabet, char* sendr,
       {
         // With binary messages udh is NOT taken from the PDU.
         i = with_udh;
-        if (*alphabet == 1)
-          i = 0;
+        // 3.1.5: it should work now:
+        if (bin_udh == 0)
+          if (*alphabet == 1)
+            i = 0;
         if ((result = pdu2binary(Src_Pointer, message, message_length, expected_length, i, udh_data, udh_type, &errorpos)) < 0)
           pdu_error(err_str, 0, Src_Pointer -full_pdu +errorpos, 0, "While reading TP-UD (%s): %s",
                     (*alphabet == 1)? "binary" : "UCS2 text",
@@ -1161,7 +1241,7 @@ int split_type_2(char *full_pdu, char* Src_Pointer,char* sendr, char* date,char*
                 {
                   // Not a fatal error (?)
                   //pdu_error(err_str, 0, Src_Pointer -full_pdu, Length +padding, "Invalid character(s) in recipient address: \"%s\"", sendr);
-                  //*sendr = 0;
+                  // *sendr = 0;
                   add_warning(warning_headers, "Invalid character(s) in recipient address.");
                   break;
                 }
@@ -1186,7 +1266,7 @@ int split_type_2(char *full_pdu, char* Src_Pointer,char* sendr, char* date,char*
               {
                 // Not a fatal error (?)
                 //pdu_error(err_str, 0, Src_Pointer -full_pdu, 6, "Invalid value(s) in date of SMSC Timestamp: \"%s\"", date);
-                //*date = 0;
+                // *date = 0;
                 add_warning(warning_headers, "Invalid value(s) in date of SMSC Timestamp.");
               }
 
@@ -1201,7 +1281,7 @@ int split_type_2(char *full_pdu, char* Src_Pointer,char* sendr, char* date,char*
               {
                 // Not a fatal error (?)
                 //pdu_error(err_str, 0, Src_Pointer -full_pdu, 6, "Invalid value(s) in time of SMSC Timestamp: \"%s\"", time);
-                //*time = 0;
+                // *time = 0;
                 add_warning(warning_headers, "Invalid value(s) in time of SMSC Timestamp.");
               }
 
@@ -1322,7 +1402,8 @@ int split_type_2(char *full_pdu, char* Src_Pointer,char* sendr, char* date,char*
 // Returns the length of the message 
 int splitpdu(char *pdu, char *mode, int *alphabet, char *sendr, char *date, char *time, char *message,
              char *smsc, int *with_udh, char *a_udh_data, char *a_udh_type, int *is_statusreport,
-             int *is_unsupported_pdu, char *from_toa, int *report, int *replace, char *warning_headers, int *flash)
+             int *is_unsupported_pdu, char *from_toa, int *report, int *replace, char *warning_headers,
+             int *flash, int bin_udh)
 {
   int Length;
   int Type;
@@ -1451,7 +1532,7 @@ int splitpdu(char *pdu, char *mode, int *alphabet, char *sendr, char *date, char
                       {
                         // Not a fatal error (?)
                         //pdu_error(&err_str, 0, Pointer -pdu, Length, "Invalid character(s) in numeric SMSC address: \"%s\"", smsc);
-                        //*smsc = 0;
+                        // *smsc = 0;
                         add_warning(warning_headers, "Invalid character(s) in numeric SMSC address");
                         break;
                       }
@@ -1490,7 +1571,7 @@ int splitpdu(char *pdu, char *mode, int *alphabet, char *sendr, char *date, char
             {
               Pointer += 2;
               result = split_type_0(pdu, Pointer, alphabet, sendr, date, time, message, &message_length, &expected_length,
-                                    *with_udh, udh_data, udh_type, from_toa, replace, &err_str, warning_headers, flash);
+                                    *with_udh, udh_data, udh_type, from_toa, replace, &err_str, warning_headers, flash, bin_udh);
 
               if (err_str && *udh_type)
                 pdu_error(&err_str, "", -1, 0, "Message has Udh_type: %s", udh_type);
@@ -1592,3 +1673,119 @@ int splitpdu(char *pdu, char *mode, int *alphabet, char *sendr, char *date, char
 
   return result;
 }
+
+int get_pdu_details(char *dest, int size_dest, char *pdu, int mnumber)
+{
+  int result = 0;
+  int udlen;
+  int alphabet;
+  char sender[100];
+  char date[9];
+  char time[9];
+  char ascii[MAXTEXT];
+  char smsc[31];
+  int with_udh;
+  char udh_data[SIZE_UDH_DATA];
+  char udh_type[SIZE_UDH_TYPE];
+  int is_statusreport;
+  int is_unsupported_pdu;
+  char from_toa[51];
+  int report;
+  int replace;
+  char warning_headers[SIZE_WARNING_HEADERS];
+  int flash;
+  int bin_udh = 1;
+  int m_id = 999; // real id for concatenated messages only, others use this.
+  int length_sender = 32;
+  int p_count = 1;
+  int p_number = 1;
+  char buffer[100]; // 3.1.6: increased size, was too small (51).
+  int i;
+  char sort_ch;
+  char *p;
+
+  udlen = splitpdu(pdu, DEVICE.mode, &alphabet, sender, date, time, ascii, smsc, &with_udh, udh_data, udh_type,
+                   &is_statusreport, &is_unsupported_pdu, from_toa, &report, &replace, warning_headers, &flash, bin_udh);
+
+  if (is_unsupported_pdu)
+  {
+    writelogfile(LOG_ERR, 1, "Message %i, unsupported PDU.", mnumber);
+    result = 1;
+  }
+  else
+  {
+    if (with_udh)
+    {
+      if (get_remove_concatenation(udh_data, &m_id, &p_count, &p_number) < 0)
+      {
+        writelogfile(LOG_ERR, 1, "Message %i, error while checking UDH_DATA.", mnumber);
+        result = 2;
+      }
+    }
+
+    if (!result)
+    {
+      if (strlen(sender) > length_sender)
+      {
+        writelogfile(LOG_ERR, 1, "Message %i, too long sender field.", mnumber);
+        result = 3;
+      }    
+
+      while (strlen(sender) < length_sender)
+        strcat(sender, " ");
+
+      if (DEVICE.priviledged_numbers[0])
+        p = DEVICE.priviledged_numbers;
+      else
+      if (priviledged_numbers[0])
+        p = priviledged_numbers;
+      else
+        p = 0;
+
+      sort_ch = 'Z';
+      if (p)
+      {
+        i = 0;
+        while (*p)
+        {
+          if (strncmp(sender, p, strlen(p)) == 0)
+          {
+            sort_ch = 'A' +i;
+            break;
+          }
+          i++;
+          p = strchr(p, 0) +1;
+        }
+      }
+/*
+001 A 358401234567____________________111 001/001 r 00-00-00 00-00-00n
+*/
+      sprintf(buffer, "%.03i %c %s%.03i %.03i/%.03i %c %-8.8s %-8.8s\n",
+              mnumber, sort_ch, sender, m_id, p_number, p_count, (is_statusreport)? 'r' : 'i', date, time);
+
+      if (strlen(dest) +strlen(buffer) < size_dest)
+        strcat(dest, buffer);
+      else
+      {
+        writelogfile(LOG_ERR, 1, "Message %i, not enough storage space.", mnumber);
+        result = 4;
+      }
+    }
+  }
+
+  return result;
+}
+
+int sort_pdu_helper(const void *a, const void *b)
+{
+  return(strncmp((char *)a +4, (char *)b +4, 45));
+}
+
+void sort_pdu_details(char *dest)
+{
+  int count;
+
+  count = strlen(dest) / LENGTH_PDU_DETAIL_REC;
+  if (count > 1)
+    qsort((void *)dest, count, LENGTH_PDU_DETAIL_REC, sort_pdu_helper);
+}
