@@ -1,6 +1,6 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
@@ -16,14 +16,21 @@ Either version 2 of the License, or (at your option) any later version.
 #ifndef SMSD_CFG_H
 #define SMSD_CFG_H
 
+#include <limits.h>
+#include <sys/types.h>
+#include <time.h>
+
 #define DATETIME_DEFAULT "%y-%m-%d %H:%M:%S"
 
 #define CONCATENATED_DIR_FNAME "%s/%s-concatenated"
 
-#define PROVIDER 16 
-#define DEVICES 64
+#define MM_CORE_FNAME "/tmp/mm_smsd_%i" /* %i is PID */
+
 #define NUMS 64
 
+#define DEVICE devices[process_id]
+#define STATISTICS statistics[process_id]
+
 // Maximum size of a message text
 #define MAXTEXT 39016
 
@@ -45,14 +52,39 @@ Either version 2 of the License, or (at your option) any later version.
 #define SIZE_MESSAGEIDS 4096
 #define SIZE_IDENTITY 100
 #define SIZE_TB 1024
+#define SIZE_CHECK_MEMORY_BUFFER 16384
+#define SIZE_LOG_LINE 16384
+#define SIZE_PRIVILEDGED_NUMBERS 512
+#define SIZE_SMSD_DEBUG 100
+#define SIZE_SHARED_BUFFER 256
 
-#include <limits.h>
-#include <sys/types.h>
+// Check memory methods:
+#define CM_NO_CPMS 0
+#define CM_S_NO_CPMS "Fixed values are used as CPMS does not work."
+#define CM_CPMS 1
+#define CM_S_CPMS "CPMS is used."
+#define CM_CMGD 2
+#define CM_S_CMGD "CMGD is used."
+#define CM_CMGL 3
+#define CM_S_CMGL "CMGL is used."
+#define CM_CMGL_DEL_LAST 4
+#define CM_S_CMGL_DEL_LAST "CMGL is used and messages are deleted after all messsages are read."
+#define CM_CMGL_CHECK 31
+#define CM_S_CMGL_CHECK "CMGL is used and messages are taken from the list."
+#define CM_CMGL_DEL_LAST_CHECK 41
+#define CM_S_CMGL_DEL_LAST_CHECK "CMGL is used and messages are taken from the list, messages are deleted after all messages are read."
+#define CM_CMGL_SIMCOM 5
+#define CM_S_CMGL_SIMCOM "CMGL is used. SIM600 compatible, see the manual for details."
+
+#define LENGTH_PDU_DETAIL_REC 70
 
 char process_title[32];         // smsd for main task, name of a modem for other tasks.
 int process_id;                 // -1 for main task, all other have numbers starting with 0.
                                 // This is the same as device, can be used like devices[process_id]...
-int modem;                      // Handle for modem.
+
+time_t process_start_time;
+
+int modem_handle;               // Handle for modem.
 
 int put_command_timeouts;
 
@@ -66,11 +98,12 @@ typedef struct
 typedef struct
 {
   char name[32];		// Name of the modem
+  char number[32];              // 3.1.4: SIM card's telephone number.
   char device[PATH_MAX];	// Serial port name
   char identity[SIZE_IDENTITY]; // Identification asked from the modem (CIMI)
   char conf_identity[SIZE_IDENTITY]; // Identification set in the conf file (CIMI)
   //char identity_header[SIZE_TO];// Title of current identification (IMSI)
-  char queues[PROVIDER][32];	// Assigned queues
+  char queues[NUMBER_OF_MODEMS][32]; // Assigned queues
   int incoming; 		// Try to receive SMS. 0=No, 1=Low priority, 2=High priority
   int outgoing;                 // = 0 if a modem is not used to send messages.
   int report; 			// Ask for delivery report 0 or 1 (experimental state)
@@ -97,6 +130,7 @@ typedef struct
                                 // as it's opened and closed. You can use some regular file for this. Must be an existing file.
   int decode_unicode_text;      // 1 if unicode text is decoded internally.
   int internal_combine;         // 1 if multipart message is combined internally.
+  int internal_combine_binary;  // 1 if multipart binary message is combined internally. Defaults to internal_combine.
   int pre_init;                 // 1 if pre-initialization is used with a modem.
   int check_network;            // 0 if a modem does not support AT+CREG command.
   char admin_to[SIZE_TO];       // Destination number for administrative messages.
@@ -114,6 +148,25 @@ typedef struct
   int loglevel;                 // Log Level (9=highest). Verbosity of log file.
   int messageids;               // Defines how message id's are stored: 1 = first, 2 = last (default), 3 = all.
   int voicecall_vts_list;       // Defines how VTS command is sent: 1 = as a list like "1,2,3,4", 2 = single note with one VTS command (default).
+  int voicecall_ignore_modem_response; // Delay defined with TIME: is not breaked even if modem gives some response.
+  int voicecall_hangup_ath;     // If ATH is used instead of AT+CHUP.
+  int check_memory_method;      // 0 = CPMS not supported, 1 = CPMS supported and must work (default), 2 = CMGD used to check messages, 3 = CMGL is used.
+  char cmgl_value[32];          // With check_memory_method 3, correct value for AT+CMGL= must be given here.
+  char priviledged_numbers[SIZE_PRIVILEDGED_NUMBERS]; // Priviledged numbers in incoming messages.
+  int read_timeout;             // Timeout for reading from a modem, in seconds.
+  int ms_purge_hours;           // Wich check_memory_method 5 (SIM600), messages with missing part(s) are removed from a
+  int ms_purge_minutes;         // modem after timeout defined with these two settings. Both values 0 disables this feature.
+  int ms_purge_read;            // 1 if available parts are read when purge timeout is reached. 0 if parts are only deleted.
+  int detect_message_routing;   // 0 if CMT/CDS detection is disabled.
+  int detect_unexpected_input;  // 0 if if detection is disabled.
+  int unexpected_input_is_trouble; // 0 if unexpected input / routed message should NOT activate trouble.log
+  int adminmessage_limit;       // Limit counter for administrative alert messages. 0 = no limit.
+  int adminmessage_count_clear; // Period to automatically clear administrative alert counter. Value is MINUTES. 
+  int status_signal_quality;    // 1 = signal quality is written to status file.
+  int status_include_counters;  // 1 = succeeded, failed and received counters are included in the status line.
+  int communication_delay;      // Time between each put_command (milliseconds), some modems need this.
+  int hangup_incoming_call;     // 1 = if detected unexpected input contains RING and we want to end call.
+  int max_continuous_sending;   // Defines when sending is breaked to do check/do other tasks. Time in seconds.
 } _device;
 
 // NOTE for regular run intervals: effective value is at least delaytime.
@@ -133,9 +186,10 @@ char checkhandler[PATH_MAX];    // Handler that checks if the sms file is valid.
 int alarmlevel;			// Alarm Level (9=highest). Verbosity of alarm handler.
 char logfile[PATH_MAX];		// Name or Handle of Log File
 int  loglevel;			// Log Level (9=highest). Verbosity of log file.
-_queue queues[PROVIDER];	// Queues
-_device devices[DEVICES];	// Modem devices
+_queue queues[NUMBER_OF_MODEMS]; // Queues
+_device devices[NUMBER_OF_MODEMS]; // Modem devices
 int delaytime;			// sleep-time after workless
+int delaytime_mainprocess;      // sleep-time after workless, main process. If -1, delaytime is used.
 int blocktime;			// sleep-time after multiple errors
 int blockafter;                 // Block modem after n errors
 int errorsleeptime;		// sleep-time after each error
@@ -146,6 +200,7 @@ int store_sent_pdu;             // 0=no, 1=failed pdu's only, 2=failed and 8bit/
 int validity_period;            // Validity period for messages.
 int decode_unicode_text;        // 1 if unicode text is decoded internally.
 int internal_combine;           // 1 if multipart message is combined internally.
+int internal_combine_binary;    // 1 if multipart binary message is combined internally. Defaults to internal_combine.
 int keep_filename;              // 0 if unique filename is created to each directory when a message file is moved.
 int store_original_filename;    // 1 if an original filename is saved to message file when it's moved from
                                 // outgoing directory to spooler. Works together with keep_filename.
@@ -159,6 +214,27 @@ int outgoing_utf8;              // 1 if outgoing files are automatically convert
 int log_charconv;               // 1 if character set conversion is logged.
 int log_single_lines;           // 1 if linefeeds are removed from the modem response to be logged.
 int executable_check;           // 0 if eventhandler and other executables are NOT checked during the startup checking.
+int keep_messages;              // For testing purposes: messages are not deleted and smsd stops after first run.
+char priviledged_numbers[SIZE_PRIVILEDGED_NUMBERS]; // Priviledged numbers in incoming messages.
+int ic_purge_hours;             // If internal_combine is used, concatenation storage is checked every ic_purge_interval minutes
+int ic_purge_minutes;           // and if there is message parts older than defined, they are handled or deleted.
+int ic_purge_read;              // 1 = message parts are stored as single messages. 0 = parts are just deleted.
+int ic_purge_interval;          // 
+char shell[PATH_MAX];           // Shell used to run eventhandler, defaults to /bin/sh
+char adminmessage_device[32];   // Name of device used to send administrative messages of mainspooler.
+int smart_logging;              // 1 = if loglevel is less than 7, degug log is written is there has been any errors.
+int status_signal_quality;      // 1 = signal quality is written to status file.
+int status_include_counters;    // 1 = succeeded, failed and received counters are included in the status line.
+int hangup_incoming_call;       // 1 = if detected unexpected input contains RING and we want to end call.
+int max_continuous_sending;     // Defines when sending is breaked to do check/do other tasks. Time in minutes.
+int voicecall_hangup_ath;       // If ATH is used instead of AT+CHUP.
+
+// Undocumented:
+int trust_outgoing;             // 1 = it's _sure_ that files are created by rename AND permissions are correct. Speeds up spooling.
+
+// Undocumented:
+int ignore_outgoing_priority;   // 1 = Priority: high header is not checked. Speeds up spooling.
+
 int message_count;              // Counter for sent messages. Multipart message is one message.
 
 int terminate;                  // The current process terminates if this is 1
@@ -178,11 +254,14 @@ char arg_logfile[PATH_MAX];
 int arg_terminal;
 
 int terminal;                   // 1 if smsd is communicating with terminal.
-pid_t device_pids[DEVICES];     // Pid's of modem processes.
+pid_t device_pids[NUMBER_OF_MODEMS]; // Pid's of modem processes.
 char run_info[PATH_MAX];        // Information about external script/program execution.
 
 char communicate[32];           // Device name for terminal communication mode.
 
+char international_prefixes[PATH_MAX +1];
+char national_prefixes[PATH_MAX +1];
+
 // Storage for startup errors:
 char *startup_err_str;
 int startup_err_count;
@@ -190,6 +269,7 @@ int startup_err_count;
 // Storage for PDU's:
 char *incoming_pdu_store;
 char *outgoing_pdu_store;
+char *routed_pdu_store;
 
 // Storage for getfile errors:
 char *getfile_err_store;
@@ -197,6 +277,9 @@ char *getfile_err_store;
 // Text buffer for error messages:
 char tb[SIZE_TB];
 
+// Buffer for SIM memory checking:
+char check_memory_buffer[SIZE_CHECK_MEMORY_BUFFER];
+
 int os_cygwin;                  // 1 if we are on Cygwin.
 
 char language_file[PATH_MAX];   // File name of translated headers.
@@ -207,6 +290,10 @@ char no_word[SIZE_HEADER];      // "no"
 char datetime_format[SIZE_HEADER]; // strftime format string for time stamps (not inside status reports).
 int translate_incoming;         // 0 if incoming message headers are NOT transtaled.
 
+// Next two are ffor debugging purposes:
+int enable_smsd_debug;
+char smsd_debug[SIZE_SMSD_DEBUG]; // Header of an outgoing message file.
+
 /* initialize all variable with default values */
 
 void initcfg();
